"""Simplified internal data parsing and transforming"""

from .data import filtered_data_split, __data_split, data_kwargs, flatmesh
__all__ = ['filtered_data_split', '__data_split', 'data_kwargs', 'flatmesh']
